import gzip
import base64

DEFAULT_ENCODING = 'utf-8'

def from_base64(contents, encoding = DEFAULT_ENCODING, **kwargs):
    contents = base64.b64decode(contents.encode(encoding), validate = True)
    return from_bytes(contents, encoding = encoding, **kwargs)

def from_bytes(content, string = False, encoding = DEFAULT_ENCODING):
    data = gzip.decompress(content)

    if (not string):
        return data

    return data.decode(encoding)

def to_base64(path, encoding = DEFAULT_ENCODING):
    data = to_bytes(path)
    content = base64.b64encode(data)
    return content.decode(encoding)

def to_bytes(path):
    with open(path, 'rb') as file:
        data = file.read()

    return gzip.compress(data)



import os
import shutil

import autograder.util.dirent

def archive_dir(path, **kwargs):
    """
    Create an archive of the given dir in a temp directory,
    and return the path to the created archive.
    All kwargs will be passed to get_temp_path().
    """

    if (not os.path.isdir(path)):
        raise ValueError("Target archive path does not exist or is not a dir: '%s'." % (path))

    temp_dir = autograder.util.dirent.get_temp_path(**kwargs)
    os.makedirs(temp_dir, exist_ok = True)

    base_name = os.path.basename(path)
    base_path = os.path.join(temp_dir, base_name)

    return shutil.make_archive(base_path, 'zip', path)



import sys

import autograder.api.courses.upsert.filespec
import autograder.cli.courses.upsert.common

def run(arguments):
    arguments = vars(arguments)

    filespec = _build_filespec(arguments)
    arguments['filespec'] = filespec

    results = autograder.api.courses.upsert.filespec.send(arguments, exit_on_error = True)

    return autograder.cli.courses.upsert.common.handle_results(results,
        arguments['full_output'])

def _build_filespec(arguments):
    filespec = {
        'type': arguments['type'],
        'path': arguments['path'],
    }

    for key in ['reference', 'username', 'token']:
        value = arguments.get(key, None)
        if (value is None):
            continue

        filespec[key] = value

    return filespec

def main():
    return run(_get_parser().parse_args())

def _get_parser():
    parser = autograder.api.courses.upsert.filespec._get_parser()

    autograder.cli.courses.upsert.common.add_full_output_argument(parser)

    parser.add_argument('--type', dest = 'type',
        action = 'store', type = str, required = True,
        choices = ['git', 'url'],
        help = 'The type of filespec.')

    parser.add_argument('--path', dest = 'path',
        action = 'store', type = str, required = True,
        help = 'The path the filespec points to.')

    parser.add_argument('--reference', dest = 'reference',
        action = 'store', type = str, default = None,
        help = 'The reference (often git commit/branch) of the filespec.')

    parser.add_argument('--username', dest = 'username',
        action = 'store', type = str, default = None,
        help = 'The username for filespec authentication.')

    parser.add_argument('--token', dest = 'token',
        action = 'store', type = str, default = None,
        help = 'The token for filespec authentication.')

    # Note that dest is not used for course upserting (the name of the dirs do not matter).

    return parser

if (__name__ == '__main__'):
    sys.exit(main())



import json
import sys

import autograder.api.courses.admin.update

def run(arguments):
    result = autograder.api.courses.admin.update.send(arguments, exit_on_error = True)
    result = result['result']

    exit_status = 0
    if (not result['success']):
        exit_status = 1

    if (arguments.full_output):
        print(json.dumps(result, indent = 4))
        return exit_status

    if (result['success']):
        print("Course updated.")
    else:
        print("Course not updated.")
        print("Message from server: '%s'." % (result.get('message', '')))

    return exit_status

def main():
    return run(_get_parser().parse_args())

def _get_parser():
    parser = autograder.api.courses.admin.update._get_parser()

    parser.add_argument('--full-output', dest = 'full_output',
        action = 'store_true', default = False,
        help = 'See the full course update output (as JSON) (default: %(default)s).')

    return parser

if (__name__ == '__main__'):
    sys.exit(main())



import os
import sys

import autograder.api.courses.upsert.zip
import autograder.cli.courses.upsert.common
import autograder.error
import autograder.util.zip

def run(arguments):
    try:
        zip_path = _prep_zip(arguments.path)
    except autograder.error.AutograderError as ex:
        print(str(ex))
        return 1

    results = autograder.api.courses.upsert.zip.send(arguments, files = [zip_path],
        exit_on_error = True)

    return autograder.cli.courses.upsert.common.handle_results(results,
        arguments.full_output)

def _prep_zip(path):
    if (not os.path.exists(path)):
        raise autograder.error.AutograderError("Path does not exist: '%s'." % (path))

    if (os.path.isfile(path)):
        extension = os.path.splitext(path)[1]
        if (extension != '.zip'):
            raise autograder.error.AutograderError(
                "Expecting '.zip' extension, found '%s': '%s'." % (extension, path))

        return path

    return autograder.util.zip.archive_dir(path)

def main():
    return run(_get_parser().parse_args())

def _get_parser():
    parser = autograder.api.courses.upsert.zip._get_parser()

    autograder.cli.courses.upsert.common.add_full_output_argument(parser)

    parser.add_argument('path', metavar = 'PATH',
        action = 'store', type = str,
        help = ('The path to your course material.'
            + ' Either a zip file (with .zip extension)'
            + ' or durectory (that will get zipped)'))

    return parser

if (__name__ == '__main__'):
    sys.exit(main())



import sys

import autograder.api.courses.users.list
import autograder.cli.common
import autograder.cli.config

def run(arguments):
    result = autograder.api.courses.users.list.send(arguments, exit_on_error = True)
    autograder.cli.common.list_users(result['users'], True, table = arguments.table)
    return 0

def main():
    return run(_get_parser().parse_args())

def _get_parser():
    parser = autograder.api.courses.users.list._get_parser()

    autograder.cli.config.add_table_argument(parser)

    return parser

if (__name__ == '__main__'):
    sys.exit(main())



import json

def handle_results(results, full_output = False):
    results = results['results']

    if (len(results) == 0):
        print("No courses found.")
        return 2

    error_count = 0
    for result in results:
        success = result['success']
        course_id = result['course-id']

        if (not success):
            error_count += 1

        if (full_output):
            continue

        if (not success):
            print("Course '%s' not updated." % (course_id))
            print("Message from server: '%s'." % (result.get('message', '')))
        elif (result.get('created', False)):
            print("Course '%s' created." % (course_id))
        else:
            print("Course '%s' updated." % (course_id))

    if (full_output):
        print(json.dumps(results, indent = 4))

    return error_count

def add_full_output_argument(parser):
    parser.add_argument('--full-output', dest = 'full_output',
        action = 'store_true', default = False,
        help = 'See the full course update output (as JSON) (default: %(default)s).')



import json
import sys

import autograder.api.metadata.describe

def run(arguments):
    result = autograder.api.metadata.describe.send(arguments, exit_on_error = True)

    print(json.dumps(result, indent = 4))

    return 0

def main():
    return run(_get_parser().parse_args())

def _get_parser():
    parser = autograder.api.metadata.describe._get_parser()

    return parser

if (__name__ == '__main__'):
    sys.exit(main())



import os
import unittest

import autograder.util.dirent
import autograder.util.gzip

THIS_DIR = os.path.abspath(os.path.dirname(os.path.realpath(__file__)))
DATA_DIR = os.path.join(THIS_DIR, "data")

class TestGzip(unittest.TestCase):
    def test_base(self):
        input_path = os.path.join(DATA_DIR, 'hw0_solution.py')
        with open(input_path, 'rb') as file:
            raw_data = file.read()

        compressed_data = autograder.util.gzip.to_base64(input_path)
        output_data = autograder.util.gzip.from_base64(compressed_data)

        self.assertEqual(raw_data, output_data, 'Decompressed data does not match raw file data.')
